package net.servodata.beei;

import net.servodata.beei.data.Correlatable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.switchyard.Exchange;
import org.switchyard.ExchangeInterceptor;
import org.switchyard.HandlerException;
import org.switchyard.Scope;
import org.switchyard.component.bpm.BPMConstants;

import javax.inject.Named;
import java.util.Arrays;
import java.util.List;

/**
 * @author Servodata a.s. <www.servodata.net>
 */
@Named
public class CorrelationKeyInterceptor implements ExchangeInterceptor {

	private static final Logger LOG = LoggerFactory.getLogger(CorrelationKeyInterceptor.class);

	@Override
	public List<String> getTargets() {
		return Arrays.asList(PROVIDER);
	}

	@Override
	public void after(String target, Exchange exchange) throws HandlerException {
		//NOOP
	}

	/**
	 * Sets up correlation key property before bpm component is called.
	 *
	 * @param target
	 * @param exchange
	 * @throws org.switchyard.HandlerException
	 */
	@Override
	public void before(String target, Exchange exchange) throws HandlerException {
		if (exchange.getProvider() != null && "SimplePromotedService".equals(exchange.getProvider().getName().getLocalPart())) {
			if (exchange.getMessage().getContent() instanceof org.switchyard.HandlerException) {
				System.out.println();
				return;
			}
			Correlatable correlatable = exchange.getMessage().getContent(Correlatable.class);

			LOG.info("Intercepted: {}, correlationKey {}", correlatable, correlatable.getCorrelationKey());
			exchange.getContext().setProperty(BPMConstants.CORRELATION_KEY_PROPERTY, correlatable.getCorrelationKey(), Scope.EXCHANGE);
		}
	}
}
